use super::helpers;
use wccg_models::activity::RiderSummary;
use wccg_models::activity::{ActivityType, Stats};
use wccg_models::any::Any;
use wread_mongodb::crud_repository;
use wread_mongodb::mongodb::{bson::doc, error::Error, Database};

const COLLECTION_NAME: &str = "riderSummary";

pub async fn get_by_rider_id(rider_id: &str, db: &Database) -> Result<Option<RiderSummary>, Error> {
    crud_repository::find_one(doc! {"riderId": rider_id }, COLLECTION_NAME, &db).await
}

pub async fn increment_activity_stats(
    rider_id: &str,
    activity_type: &ActivityType,
    stats: &Stats,
    db: &Database,
) {
    log::trace!("increment_activity_totals");
    init_rider_summary(rider_id, db).await;

    if let Some(update_document) = helpers::get_activity_stats_update_document(activity_type, stats)
    {
        crud_repository::update_one(
            doc! {"riderId": rider_id},
            update_document,
            None,
            COLLECTION_NAME,
            db,
        )
        .await
        .unwrap();
    } else {
        log::trace!(
            "None from get_activity_stats_update_document for {:?}",
            &activity_type
        );
    }
}

async fn init_rider_summary(rider_id: &str, db: &Database) {
    let rider_summary =
        crud_repository::find_one::<Any>(doc! {"riderId": rider_id}, COLLECTION_NAME, db)
            .await
            .unwrap();

    if let None = rider_summary {
        let empty_rider_summary = RiderSummary::new(rider_id.into());
        crud_repository::add(&empty_rider_summary, COLLECTION_NAME, db)
            .await
            .unwrap();
    }
}
