use wccg_models::event::RegistrationOption;
use wread_mongodb::crud_repository;
use wread_mongodb::mongodb::bson::{doc, oid::ObjectId};
use wread_mongodb::mongodb::error::Error;
use wread_mongodb::mongodb::options::ReplaceOptions;
use wread_mongodb::mongodb::results::{DeleteResult, InsertOneResult, UpdateResult};
use wread_mongodb::mongodb::Database;

const COLLECTION_NAME: &str = "registrationOptions";

pub async fn get(id: &str, db: &Database) -> Result<Option<RegistrationOption>, Error> {
    let id_object = ObjectId::with_string(id).unwrap();
    crud_repository::find_by_id(&id_object, COLLECTION_NAME, &db).await
}

pub async fn get_by_registration_option_id(
    event_id: &str,
    registration_option_id: &str,
    db: &Database,
) -> Result<Vec<RegistrationOption>, Error> {
    let filter = doc! {"eventId": event_id, "registrationOptionId": registration_option_id };
    crud_repository::find(filter, COLLECTION_NAME, &db).await
}

pub async fn get_all_active_auto_approval(db: &Database) -> Result<Vec<RegistrationOption>, Error> {
    let filter = doc! {"tracking.autoApprovalActive": true};
    crud_repository::find(filter, COLLECTION_NAME, &db).await
}

pub async fn add(option: &RegistrationOption, db: &Database) -> Result<InsertOneResult, Error> {
    crud_repository::add(option, COLLECTION_NAME, db).await
}

pub async fn delete(id: &str, db: &Database) -> Result<DeleteResult, Error> {
    let id_object = ObjectId::with_string(id).unwrap();
    let query = doc! {"_id": id_object };
    crud_repository::delete_one(query, None, COLLECTION_NAME, &db).await
}

pub async fn update(option: RegistrationOption, db: &Database) -> Result<UpdateResult, Error> {
    let id = option.id().clone().unwrap();
    let filter = doc! {"_id":  id};
    crud_repository::replace_one(filter, &option, None, COLLECTION_NAME, db).await
}

pub async fn upsert(option: RegistrationOption, db: &Database) -> Result<UpdateResult, Error> {
    let query = doc! {"eventId": option.event_id(), "registrationOptionId": option.registration_option_id()};
    let replace_options = ReplaceOptions::builder().upsert(true).build();
    crud_repository::replace_one(query, &option, Some(replace_options), COLLECTION_NAME, db).await
}
